"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.runGenerator = exports.registerGenerators = exports.resolveData = exports.initialiseInputs = exports.printDataResolutionResult = exports.validateInputs = exports.printHelp = exports.validateData = exports.getSeed = exports.disablePrompts = exports.extractSeedData = exports.extractSeedConfig = exports.extractPromptData = exports.removeUndefined = void 0;
const tslib_1 = require("tslib");
const fs_1 = require("fs");
const path_1 = require("path");
const foundation_cli_1 = require("@genesislcap/foundation-cli");
const DSL_MARKER = 'gpl';
const removeUndefined = (obj) => {
    Object.keys(obj).forEach((key) => (obj[key] === undefined ? delete obj[key] : {}));
    return obj;
};
exports.removeUndefined = removeUndefined;
const extractPromptData = (prompts, previousAnswers) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
    const seedInputs = [];
    const inquirer = {
        prompt: (inputs) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
            seedInputs.push(...inputs);
            return previousAnswers;
        }),
    };
    const answers = yield prompts()(inquirer, previousAnswers);
    const defaults = seedInputs.reduce((acc, value) => {
        let addDefault;
        if (typeof value.when === 'function') {
            addDefault = value.when(acc);
        }
        else if (typeof value.when === 'undefined') {
            addDefault = true;
        }
        else {
            addDefault = value.when;
        }
        if (addDefault) {
            acc[value.name] = value.default;
        }
        return acc;
    }, {});
    const validators = seedInputs.reduce((acc, value) => {
        acc[value.name] = value.validate;
        return acc;
    }, {});
    (0, exports.removeUndefined)(previousAnswers);
    return {
        defaults: defaults,
        answers: (0, exports.removeUndefined)(answers),
        validators: (0, exports.removeUndefined)(validators),
    };
});
exports.extractPromptData = extractPromptData;
const extractSeedConfig = (seedDir) => {
    const seedConfig = (0, foundation_cli_1.getSeedConfigFile)(seedDir);
    const details = typeof seedConfig.details === 'function' ? seedConfig.details() : {};
    return { seedConfig, details: (0, exports.removeUndefined)(details) };
};
exports.extractSeedConfig = extractSeedConfig;
const extractSeedData = (seedDir, previousAnswers) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
    const { seedConfig, details } = (0, exports.extractSeedConfig)(seedDir);
    const genxDir = (0, path_1.resolve)(seedDir, '.genx');
    yield foundation_cli_1.NpmActions.install(genxDir, true);
    const { defaults, answers, validators } = yield (0, exports.extractPromptData)(seedConfig.prompts, previousAnswers);
    return {
        defaults,
        answers,
        validators,
        details,
    };
});
exports.extractSeedData = extractSeedData;
const disablePrompts = (generator) => (Object.assign(Object.assign({}, generator), { prompts: [] }));
exports.disablePrompts = disablePrompts;
const getSeed = (details, seedDirPath) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
    const [appSeed] = yield (0, foundation_cli_1.seedNormalizer)([
        {
            type: foundation_cli_1.SeedSource.local,
            name: details.name || 'Local Seed',
            about: details.description || '',
            baseDir: seedDirPath,
            tags: [],
            configure: null,
        },
    ]);
    return appSeed;
});
exports.getSeed = getSeed;
const validateData = (combined, validators) => {
    const validationErrors = [];
    Object.keys(combined).forEach((key) => {
        const value = combined[key];
        if (typeof value === 'undefined') {
            validationErrors.push({ message: `${key} is undefined` });
        }
        else if (value === '') {
            validationErrors.push({ message: `${key} is empty` });
        }
        else if (validators[key]) {
            const isValid = validators[key](value);
            if (isValid !== true) {
                validationErrors.push({ message: `${key} is invalid: ${value} ${isValid}` });
            }
        }
    });
    return validationErrors;
};
exports.validateData = validateData;
const printHelp = () => {
    console.log(`
  Usage:
    xgen appname seedfolder
    xgen appname ./relative/path/to/seedfolder
    xgen appname seedfolder --dataParam1 value --dataParam2 value
    xgen appname seedfolder generatorname
    xgen appname seedfolder generatorname --dataParam1 value --dataParam2 value
  `);
};
exports.printHelp = printHelp;
const validateInputs = (appName, appDirectory, seedDir, seedDirPath) => {
    const targetDirectory = appDirectory && appName ? (0, path_1.join)(appDirectory, appName) : null;
    if (!appName || !seedDir) {
        (0, exports.printHelp)();
        process.exit(1);
    }
    else if (!seedDirPath || !(0, fs_1.existsSync)(seedDirPath)) {
        console.log(`Invalid seed path: ${seedDirPath}`);
        process.exit(1);
    }
    else if (targetDirectory && (0, fs_1.existsSync)(targetDirectory)) {
        console.log(`Target directory already exists: ${targetDirectory}`);
        process.exit(1);
    }
};
exports.validateInputs = validateInputs;
const printDataResolutionResult = (validationErrors, combined) => {
    console.log('---------------------------------------------------------');
    if (validationErrors.length) {
        console.log('Validation errors detected:');
        validationErrors.forEach((error) => console.log(`  * ${error.message}`));
        console.log('');
        process.exit();
    }
    else {
        console.log('resolved data (defaults + inputs):');
        console.log(combined);
    }
};
exports.printDataResolutionResult = printDataResolutionResult;
const initialiseInputs = () => {
    const { _: [appName, seedDir, generatorName] } = foundation_cli_1.argvMinimist, data = tslib_1.__rest(foundation_cli_1.argvMinimist, ["_"]);
    const appDirectory = foundation_cli_1.argvMinimist.appDirectory || process.cwd();
    const seedDirPath = appDirectory && seedDir ? (0, path_1.resolve)(appDirectory, seedDir) : null;
    return {
        appName,
        seedDir,
        generatorName,
        appDirectory,
        seedDirPath,
        data,
    };
};
exports.initialiseInputs = initialiseInputs;
const resolveData = (appName, appDirectory, seedDirPath, data) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
    const prevAnswers = Object.assign({ appName, appDirectory }, data);
    const { defaults, answers, validators, details } = yield (0, exports.extractSeedData)(seedDirPath, prevAnswers);
    const appSeed = yield (0, exports.getSeed)(details, seedDirPath);
    const combined = Object.assign(Object.assign(Object.assign(Object.assign({}, answers), defaults), prevAnswers), { appSeed });
    const validationErrors = (0, exports.validateData)(combined, validators);
    return { combined, validationErrors };
});
exports.resolveData = resolveData;
const registerGenerators = (plop, bypassArr) => {
    [foundation_cli_1.getDSLGenerator, foundation_cli_1.getAppGenerator, foundation_cli_1.getWorkspaceGenerator].forEach((getGenerator) => {
        const { name, generator } = getGenerator(plop, bypassArr, exports.disablePrompts);
        plop.setGenerator(name, generator);
    });
};
exports.registerGenerators = registerGenerators;
const runGenerator = (plop, combined, generatorName, bypassArr) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
    const generators = plop.getGeneratorList();
    const generator = generatorName || combined.appSeed.name.includes(DSL_MARKER)
        ? generators[0].name
        : generators[1].name;
    console.log(`Using generator: ${generator}`);
    yield foundation_cli_1.GeneratorActions.run(plop.getGenerator(generator), combined, plop, bypassArr);
    return;
});
exports.runGenerator = runGenerator;
